/*
 /*
 * mini web server
 * Copyright 2014, Alex Sualdea.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 *
 *  1. Redistributions of source code must retain the above copyright notice,
 *     this list of conditions and the following disclaimer.
 *  2. Redistributions in binary form must reproduce the above copyright notice,
 *     this list of conditions and the following disclaimer in the documentation
 *     and/or other materials provided with the distribution.
 *  3. The name of the author may not be used to endorse or promote products
 *     derived from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR ``AS IS'' AND ANY EXPRESS OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF
 * MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO
 * EVENT SHALL THE AUTHOR BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS;
 * OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY,
 * WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR
 * OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF
 * ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

#include <memory>
#include "HTTPStreamServer.h"

#include "Poco/Net/HTTPServerResponseImpl.h"
#include "Poco/Net/HTTPServerRequestImpl.h"
#include "Poco/Net/NetException.h"

/*** HTTPStreamServerRequestHandler ********************************************************************/


HTTPStreamServerRequestHandlerFactory::HTTPStreamServerRequestHandlerFactory()
{
}


HTTPStreamServerRequestHandlerFactory::~HTTPStreamServerRequestHandlerFactory()
{
}



/*** HTTPStreamServerRequestHandler ********************************************************************/


HTTPStreamServerRequestHandler::HTTPStreamServerRequestHandler()
{
}


HTTPStreamServerRequestHandler::~HTTPStreamServerRequestHandler()
{
}


/*** HTTPStreamServerConnection ********************************************************************/



HTTPStreamServerConnection::HTTPStreamServerConnection(const Poco::Net::StreamSocket& socket, Poco::Net::HTTPServerParams::Ptr pParams, HTTPStreamServerRequestHandlerFactory::Ptr pFactory):
	TCPServerConnection(socket),
	_pParams(pParams),
	_pFactory(pFactory)
{
	poco_check_ptr (pFactory);
}


HTTPStreamServerConnection::~HTTPStreamServerConnection()
{
}


void HTTPStreamServerConnection::run()
{
	std::string server = _pParams->getSoftwareVersion();
	Poco::Net::HTTPServerSession session(socket(), _pParams);
	while (session.hasMoreRequests())
	{
		try
		{
			Poco::Net::HTTPServerResponseImpl response(session);
			Poco::Net::HTTPServerRequestImpl request(response, session, _pParams);
			
			Poco::Timestamp now;
			response.setDate(now);
			response.setVersion(request.getVersion());
			response.setKeepAlive(_pParams->getKeepAlive() && request.getKeepAlive() && session.canKeepAlive());
			if (!server.empty())
				response.set("Server", server);
			try
			{
				std::auto_ptr<HTTPStreamServerRequestHandler> pHandler(_pFactory->createRequestHandler(request));
				if (pHandler.get())
				{
					if (request.expectContinue())
						response.sendContinue();
					
					pHandler->handleRequest(request, response, socket(), session);
					session.setKeepAlive(_pParams->getKeepAlive() && response.getKeepAlive() && session.canKeepAlive());
				}
				else
				{
					sendErrorResponse(session, Poco::Net::HTTPResponse::HTTP_NOT_IMPLEMENTED);
				}
			}
			catch (Poco::Exception)
			{
				if (!response.sent())
					sendErrorResponse(session, Poco::Net::HTTPResponse::HTTP_INTERNAL_SERVER_ERROR);
				throw;
			}
		}
		catch (Poco::Net::NoMessageException&)
		{
			break;
		}
		catch (Poco::Net::MessageException&)
		{
			sendErrorResponse(session, Poco::Net::HTTPResponse::HTTP_BAD_REQUEST);
		}
	}
}


void HTTPStreamServerConnection::sendErrorResponse(Poco::Net::HTTPServerSession& session, Poco::Net::HTTPResponse::HTTPStatus status)
{
	Poco::Net::HTTPServerResponseImpl response(session);
	response.setVersion(Poco::Net::HTTPMessage::HTTP_1_1);
	response.setStatusAndReason(status);
	response.setKeepAlive(false);
	response.send();
	session.setKeepAlive(false);
}


/*** HTTPStreamServerConnectionFactory *************************************************/





HTTPStreamServerConnectionFactory::HTTPStreamServerConnectionFactory(Poco::Net::HTTPServerParams::Ptr pParams, HTTPStreamServerRequestHandlerFactory::Ptr pFactory):
	_pParams(pParams),
	_pFactory(pFactory)
{
	poco_check_ptr (pFactory);
}


HTTPStreamServerConnectionFactory::~HTTPStreamServerConnectionFactory()
{
}


Poco::Net::TCPServerConnection* HTTPStreamServerConnectionFactory::createConnection(const Poco::Net::StreamSocket& socket)
{
	return new HTTPStreamServerConnection(socket, _pParams, _pFactory);
}





/*** HTTPStreamServer ********************************************************************/






HTTPStreamServer::HTTPStreamServer(HTTPStreamServerRequestHandlerFactory::Ptr pFactory, const Poco::Net::ServerSocket& socket, Poco::Net::HTTPServerParams::Ptr pParams):
	Poco::Net::TCPServer(new HTTPStreamServerConnectionFactory(pParams, pFactory), socket, pParams)
{
}


HTTPStreamServer::HTTPStreamServer(HTTPStreamServerRequestHandlerFactory::Ptr pFactory, Poco::ThreadPool& threadPool, const Poco::Net::ServerSocket& socket, Poco::Net::HTTPServerParams::Ptr pParams):
	TCPServer(new HTTPStreamServerConnectionFactory(pParams, pFactory), threadPool, socket, pParams)
{
}


HTTPStreamServer::~HTTPStreamServer()
{
}
