/*
 * mini web server
 * Copyright 2014, Alex Sualdea.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 *
 *  1. Redistributions of source code must retain the above copyright notice,
 *     this list of conditions and the following disclaimer.
 *  2. Redistributions in binary form must reproduce the above copyright notice,
 *     this list of conditions and the following disclaimer in the documentation
 *     and/or other materials provided with the distribution.
 *  3. The name of the author may not be used to endorse or promote products
 *     derived from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR ``AS IS'' AND ANY EXPRESS OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF
 * MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO
 * EVENT SHALL THE AUTHOR BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS;
 * OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY,
 * WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR
 * OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF
 * ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */
#ifndef _HTTP_STREAM_SERVER_H
#define _HTTP_STREAM_SERVER_H

#include "Poco/Net/Net.h"
#include "Poco/Net/TCPServer.h"
#include "Poco/Net/HTTPResponse.h"
#include "Poco/Net/HTTPServerParams.h"
#include "Poco/Net/TCPServerConnection.h"
#include "Poco/Net/HTTPServerSession.h"
#include "Poco/Net/HTTPServerRequest.h"
#include "Poco/Net/HTTPServerResponse.h"





class HTTPStreamServerRequestHandler
	/// The abstract base class for HTTPRequestHandlers 
	/// created by HTTPServer.
	///
	/// Derived classes must override the handleRequest() method.
	/// Furthermore, a HTTPRequestHandlerFactory must be provided.
	///
	/// The handleRequest() method must perform the complete handling
	/// of the HTTP request connection. As soon as the handleRequest() 
	/// method returns, the request handler object is destroyed.
	///
	/// A new HTTPStreamServerRequestHandler object will be created for
	/// each new HTTP request that is received by the HTTPServer.
{
public:
	HTTPStreamServerRequestHandler();
		/// Creates the HTTPStreamServerRequestHandler.

	virtual ~HTTPStreamServerRequestHandler();
		/// Destroys the HTTPStreamServerRequestHandler.

	virtual void handleRequest(Poco::Net::HTTPServerRequest& request, Poco::Net::HTTPServerResponse& response, Poco::Net::StreamSocket& sock, Poco::Net::HTTPServerSession& session) = 0;
		/// Must be overridden by subclasses.
		///
		/// Handles the given request.

private:
	HTTPStreamServerRequestHandler(const HTTPStreamServerRequestHandler&);
	HTTPStreamServerRequestHandler& operator = (const HTTPStreamServerRequestHandler&);
};


class HTTPStreamServerRequestHandlerFactory
	/// A factory for HTTPStreamServerRequestHandler objects.
	/// Subclasses must override the createRequestHandler()
	/// method.
{
public:
	typedef Poco::SharedPtr<HTTPStreamServerRequestHandlerFactory> Ptr;
	
	HTTPStreamServerRequestHandlerFactory();
		/// Creates the HTTPStreamServerRequestHandlerFactory.

	virtual ~HTTPStreamServerRequestHandlerFactory();
		/// Destroys the HTTPStreamServerRequestHandlerFactory.

	virtual HTTPStreamServerRequestHandler* createRequestHandler(const Poco::Net::HTTPServerRequest& request) = 0;
		/// Must be overridden by sublasses.
		///
		/// Creates a new request handler for the given HTTP request.

private:
	HTTPStreamServerRequestHandlerFactory(const HTTPStreamServerRequestHandlerFactory&);
	HTTPStreamServerRequestHandlerFactory& operator = (const HTTPStreamServerRequestHandlerFactory&);
};



class HTTPStreamServerConnection: public Poco::Net::TCPServerConnection
	/// This subclass of TCPServerConnection handles HTTP
	/// connections.
{
public:
	HTTPStreamServerConnection(const Poco::Net::StreamSocket& socket, Poco::Net::HTTPServerParams::Ptr pParams, HTTPStreamServerRequestHandlerFactory::Ptr pFactory);
		/// Creates the HTTPStreamServerConnection.

	virtual ~HTTPStreamServerConnection();
		/// Destroys the HTTPStreamServerConnection.
		
	void run();
		/// Handles all HTTP requests coming in.

protected:
	void sendErrorResponse(Poco::Net::HTTPServerSession& session, Poco::Net::HTTPResponse::HTTPStatus status);

private:
	Poco::Net::HTTPServerParams::Ptr          _pParams;
	HTTPStreamServerRequestHandlerFactory::Ptr _pFactory;
};







class HTTPStreamServerConnectionFactory: public Poco::Net::TCPServerConnectionFactory
	/// This implementation of a TCPServerConnectionFactory
	/// is used by HTTPServer to create HTTPStreamServerConnection objects.
{
public:
	HTTPStreamServerConnectionFactory(Poco::Net::HTTPServerParams::Ptr pParams, HTTPStreamServerRequestHandlerFactory::Ptr pFactory);
		/// Creates the HTTPStreamServerConnectionFactory.

	~HTTPStreamServerConnectionFactory();
		/// Destroys the HTTPStreamServerConnectionFactory.

	Poco::Net::TCPServerConnection* createConnection(const Poco::Net::StreamSocket& socket);
		/// Creates an instance of HTTPStreamServerConnection
		/// using the given StreamSocket.
	
private:
	Poco::Net::HTTPServerParams::Ptr          _pParams;
	HTTPStreamServerRequestHandlerFactory::Ptr _pFactory;
};






class HTTPStreamServer: public Poco::Net::TCPServer
	/// A subclass of TCPServer that implements a
	/// full-featured multithreaded HTTP server.
	///
	/// A HTTPStreamServerRequestHandlerFactory must be supplied.
	/// The ServerSocket must be bound and in listening state.
	///
	/// To configure various aspects of the server, a HTTPServerParams
	/// object can be passed to the constructor.
	///
	/// The server supports:
	///   - HTTP/1.0 and HTTP/1.1
	///   - automatic handling of persistent connections.
	///   - automatic decoding/encoding of request/response message bodies
	///     using chunked transfer encoding.
	///
	/// Please see the TCPServer class for information about
	/// connection and thread handling.
	///
	/// See RFC 2616 <http://www.faqs.org/rfcs/rfc2616.html> for more
	/// information about the HTTP protocol.
{
public:
	HTTPStreamServer(HTTPStreamServerRequestHandlerFactory::Ptr pFactory, const Poco::Net::ServerSocket& socket, Poco::Net::HTTPServerParams::Ptr pParams);
		/// Creates the HTTPStreamServer, using the given ServerSocket.
		///
		/// New threads are taken from the default thread pool.

	HTTPStreamServer(HTTPStreamServerRequestHandlerFactory::Ptr pFactory, Poco::ThreadPool& threadPool, const Poco::Net::ServerSocket& socket, Poco::Net::HTTPServerParams::Ptr pParams);
		/// Creates the HTTPStreamServer, using the given ServerSocket.
		///
		/// New threads are taken from the given thread pool.

	~HTTPStreamServer();
		/// Destroys the HTTPStreamServer and its HTTPStreamServerRequestHandlerFactory.
};








#endif // _HTTP_STREAM_SERVER_H
