/*
 * mini web server
 * Copyright 2014, Alex Sualdea.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 *
 *  1. Redistributions of source code must retain the above copyright notice,
 *     this list of conditions and the following disclaimer.
 *  2. Redistributions in binary form must reproduce the above copyright notice,
 *     this list of conditions and the following disclaimer in the documentation
 *     and/or other materials provided with the distribution.
 *  3. The name of the author may not be used to endorse or promote products
 *     derived from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR ``AS IS'' AND ANY EXPRESS OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF
 * MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO
 * EVENT SHALL THE AUTHOR BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS;
 * OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY,
 * WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR
 * OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF
 * ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

#include <stdio.h>
#include <stdlib.h>
#include <iostream>
#include <sstream>
#include "Poco/Net/NetException.h"
#include "Poco/Net/HTMLForm.h"
#include "Poco/Timestamp.h"
#include "Poco/Net/HTTPBasicCredentials.h"
#include "Poco/Environment.h"
#include "Poco/Process.h"
#include "Poco/Pipe.h"
#include "Poco/File.h"
#include "Poco/PipeStream.h"
#include "Poco/StringTokenizer.h"
#include "MiniWebServer.h"
#include "Settings.h"

using Poco::Net::ServerSocket;
using Poco::Net::HTTPRequestHandler;
using Poco::Net::HTTPRequestHandlerFactory;
using Poco::Net::HTTPServer;
using Poco::Net::HTTPServerRequest;
using Poco::Net::HTTPServerResponse;
using Poco::Net::HTTPServerParams;
using Poco::Net::HTTPBasicCredentials;
using Poco::URI;
using Poco::Timestamp;
using Poco::DateTimeFormatter;
using Poco::DateTimeFormat;
using Poco::ThreadPool;
using Poco::Util::ServerApplication;
using Poco::Util::Application;
using Poco::Util::Option;
using Poco::Util::OptionSet;
using Poco::Util::HelpFormatter;
using Poco::Environment;
using Poco::Process;
using Poco::StringTokenizer;

#ifdef _WIN32
	#define popen _popen
	#define pclose _pclose
#endif


#ifdef _WIN32
        #define PHPBIN "php-cgi.exe"
#else
        #define PHPBIN "/etc/alternatives/php-cgi"
#endif



// mimetypes
std::map<std::string, std::string> MiniWebServer::mimeTypes;



/** Services **/

std::string WebShellService::bash_it(const std::string& cmdline)
{
	std::string cmd_with_err = cmdline + " 2>&1";
	std::stringstream out;
	FILE* pipe = popen(cmd_with_err.c_str(), "r");
	if (pipe)
	{
		char buffer[65535];
		while(!feof(pipe))
		{
			if(fgets(buffer, 65535, pipe) != NULL)
			{
				out << buffer;
			}
		}
		pclose(pipe);
	}
	return out.str();
}



/** MiniWebServer code **/


MiniWebRequestHandlerFactory::MiniWebRequestHandlerFactory()
{
}

MiniWebRequestHandlerFactory::~MiniWebRequestHandlerFactory()
{
	svcs.removeAll();
}



HTTPStreamServerRequestHandler* MiniWebRequestHandlerFactory::createRequestHandler(const HTTPServerRequest& request)
{
	HTTPStreamServerRequestHandler* ret;
	ret = new MiniWebRequestHandler(this);
	return ret;
}


bool MiniWebRequestHandler::getAuthentication(const Poco::Net::HTTPServerRequest& request, Poco::Net::HTTPServerResponse& response)
{
	//TODO Implement authentication
	bool authen = true;
	return authen;
}

void MiniWebRequestHandler::sendErrorPage(Poco::Net::HTTPServerResponse& response, Poco::Net::HTTPResponse::HTTPStatus status)
{
	std::ostream& ostr = response.send();
	ostr << "<html><head><title>" << SERVER_NAME <<  "</title>";
	ostr << "<body><H1>" << status << " " << Poco::Net::HTTPResponse::getReasonForStatus(status) << "</H1></body></html>";
	ostr.flush();
	return;
}


void MiniWebRequestHandler::sendDocument(Poco::Net::HTTPServerResponse& response, const std::vector<std::string>& pathSegments, std::string pathTranslated)
{
	std::string filename;
	if (pathSegments.size() == 0)
	{
		filename = "index.html";
		pathTranslated += filename;
	}
	else
	{
		filename = pathSegments[pathSegments.size()-1];
	}
	size_t dpos = filename.find_last_of('.');
	std::string ext;
	if (dpos > 0)
	{
		ext =  filename.substr(dpos+1);
		response.setContentType(MiniWebServer::getMimeType(ext));
	}
	else
	{
		ext = "";
		response.setContentType("text/plain");
	}
	FILE* file = fopen(pathTranslated.c_str(), "rb");
	if (file == NULL)
	{
		sendErrorPage(response, Poco::Net::HTTPResponse::HTTP_NOT_FOUND);
	}
	else
	{
		std::ostream& ostr = response.send();
		char buf[1024];
		size_t readed;
		do
		{
			readed = fread(buf, sizeof(char), sizeof(buf), file);
			for (size_t i=0; i<readed; ++i)
				ostr.put(buf[i]);
		} while (readed > 0);
		fclose(file);
		ostr.flush();
		return;
	}
}


void MiniWebRequestHandler::sendPHP(const Poco::Net::HTTPServerRequest& request, Poco::Net::HTTPServerResponse& response, const std::vector<std::string> pathSegments, const std::string pathTranslated)
{
	// configure CGI environment
	Environment::set("REQUEST_METHOD", request.getMethod());
	if (request.getMethod() == "GET")
	{
		Environment::set("QUERY_STRING", uri.getQuery());
		Environment::set("CONTENT_TYPE", "");
		Environment::set("CONTENT_LENGTH", "");
	}
	else
	{
		Environment::set("QUERY_STRING", "");
		Environment::set("CONTENT_TYPE", request.getContentType());
		std::string contLen = "" + request.getContentLength();
		Environment::set("CONTENT_LENGTH", contLen);
	}

//	HTTPBasicCredentials credentials(request);

//	Environment::set("REMOTE_USER", credentials.getUsername());
//	Environment::set("REMOTE_PASSWORD", credentials.getPassword());
	Environment::set("PATH_TRANSLATED", pathTranslated);
	Environment::set("PATH_INFO", uri.getPath());
	Environment::set("SCRIPT_NAME", pathSegments[pathSegments.size()-1]);
	char strPort[10];

	Environment::set("SERVER_PORT", strPort);
	Environment::set("SERVER_PROTOCOL", request.getVersion());
	Environment::set("SERVER_SOFTWARE", SERVER_NAME);
	Environment::set("SERVER_NAME", SERVER_NAME);
	Environment::set("GATEWAY_INTERFACE", "CGI/1.1");
	Environment::set("REDIRECT_STATUS", "true");

	// CGI php execution
	Poco::File php_binary(PHPBIN);
	if (!php_binary.exists())
	{
		sendErrorPage(response, Poco::Net::HTTPResponse::HTTP_INTERNAL_SERVER_ERROR);
		return;
	}
	Process::Args args;
	Poco::Pipe out;
	Poco::ProcessHandle ph(Process::launch(PHPBIN, args, 0, &out, &out));
	Poco::PipeInputStream istr(out);

	// get stdout
	((Poco::Net::MessageHeader)response).read(istr);
	std::string outStr;
	int c = istr.get();
	while (c != -1)
	{
		outStr += (char) c;
		c = istr.get();
	}
	int rc = ph.wait(); // wait the php process to finish

	std::string filename = pathSegments[pathSegments.size()-1];
	size_t dpos = filename.find_last_of('.');
	std::string ext;
	if (dpos > 0)
	{
		ext =  filename.substr(dpos+1);
		response.setContentType(MiniWebServer::getMimeType(ext));
	}
	else
	{
		ext = "";
		response.setContentType("text/plain");
	}

	std::ostream& ostr = response.send();
	ostr << outStr;
	ostr.flush();
}



//
// a service request has the form:
//		http://server/servicename?param1=val1&param2=value2
//
void MiniWebRequestHandler::webService(Poco::Net::HTTPServerRequest& request, Poco::Net::HTTPServerResponse& response, Poco::Net::StreamSocket& sock, Poco::Net::HTTPServerSession& session)
{
	if (pathSegments.size() > 1)
	{
		// a web service can only have the name
		this->sendErrorPage(response, Poco::Net::HTTPResponse::HTTP_BAD_REQUEST);
	}

	std::shared_ptr<MiniWebService> svc = parent->svcs.getService(pathSegments[0]);
	if (svc != NULL)
	{
		std::map<std::string, std::string> params;
		Poco::StringTokenizer tokenizer(query, "&", StringTokenizer::TOK_TRIM);
		for (Poco::StringTokenizer::Iterator it = tokenizer.begin(); it != tokenizer.end(); ++it)
		{
			if (*it != "")
			{
				std::string name, value;
				Poco::StringTokenizer paramTok(*it, "=", Poco::StringTokenizer::TOK_TRIM);
				Poco::StringTokenizer::Iterator tok;
				tok = paramTok.begin();
				if (tok == paramTok.end())
					continue;
				name = *tok;
				++tok;
				if (tok == paramTok.end())
					continue;
				value = *tok;
				params[name] = value;
			}
		}
		svc->exec(request, params, response, sock, session);
	}
	else
	{
		this->sendErrorPage(response, Poco::Net::HTTPResponse::HTTP_NOT_FOUND);
	}
}




void MiniWebRequestHandler::handleRequest(HTTPServerRequest& request, HTTPServerResponse& response, Poco::Net::StreamSocket& sock, Poco::Net::HTTPServerSession& session)
{
	if (!this->getAuthentication(request, response))
		return;

	uri = request.getURI();
	uri.decode(uri.getRawQuery(), query);
	getPathSegmentsFromURI(uri.getPath(), pathSegments);
	std::string pathTranslated = docRoot + uri.getPath(); // TODO:: security, chroot ...
	
	if (pathSegments.size() == 0)	// it's "/"
	{
		sendDocument(response, pathSegments, pathTranslated);
	}
	else
	{
		Poco::Path p(pathTranslated);
		std::string ext = p.getExtension();
		if (ext == "php")
		{
			sendPHP(request, response, pathSegments, pathTranslated);
		}
		else
		{
			if (pathSegments.size() ==  1 && parent->svcs.existsService(pathSegments[0]))
				webService(request, response, sock, session);
			else
				sendDocument(response, pathSegments, pathTranslated);
		}
	}
}



void MiniWebRequestHandler::getPathSegmentsFromURI(const std::string& path, std::vector<std::string>& segments)
{
	std::string::const_iterator it  = path.begin();
	std::string::const_iterator end = path.end();
	std::string seg;
	while (it != end)
	{
		if (*it == '/')
		{
			if (!seg.empty())
			{
				segments.push_back(seg);
				seg.clear();
			}
		}
		else seg += *it;
		++it;
	}
	if (!seg.empty())
		segments.push_back(seg);
}











MiniWebServer::MiniWebServer() : started(false), svs(nullptr), srv(nullptr),
								_localOnly(false), requestFactory(new MiniWebRequestHandlerFactory()),
								docRoot(Settings::get_instance().docroot)
{
	loadMimeTypes("mime.conf");
}
	
MiniWebServer::~MiniWebServer()
{
	stop();
}


void MiniWebServer::addService(const std::string& svcname, std::shared_ptr<MiniWebService> svc)
{
	requestFactory->addService(svcname, svc);
}

void MiniWebServer::removeService(const std::string& svcname)
{
	requestFactory->removeService(svcname);
}

bool MiniWebServer::start(unsigned int port,  bool localOnly)
{
	_localOnly = localOnly;

	HTTPServerParams* pParams = new HTTPServerParams;
	pParams->setMaxThreads(MAX_WEB_THREADS); 
	//pParams->setThreadPriority(Poco::Thread::PRIO_NORMAL);
	pParams->setServerName(SERVER_NAME);
	pParams->setSoftwareVersion(SOFTWARE_VERSION);
	pParams->setKeepAlive(true);
	pParams->setMaxKeepAliveRequests(0);
	// set-up a server socket
	if (localOnly)
	{
		Poco::Net::SocketAddress addr(Poco::Net::IPAddress("127.0.0.1"), port);
		if (port != 0)
			svs.reset(new ServerSocket(addr));
	}
	else
	{
		if (port != 0)
			svs.reset(new ServerSocket(port));
	}

	// set-up a HTTPServer instance
	svs->setReuseAddress(true);
	srv = new HTTPStreamServer(requestFactory, *svs, pParams);

	// start the HTTPServer
	srv->start();
	if (svs != NULL)
		started = true;
	return true;
}
	


void MiniWebServer::stop()
{
	if (srv != NULL)
		srv->stop();
}


void MiniWebServer::loadMimeTypes(std::string mimeTypesFile)
{
	FILE* f = fopen(mimeTypesFile.c_str(), "rt");
	char cline[65536];
	if (f != NULL)
	{
		while(fgets(cline, 65536, f))
		{

			std::string line(Poco::trimLeft(std::string(cline)));
			if (line[0] != '#')	// if not comment
			{
				std::list<std::string> toks;
				StringTokenizer tokenizer(line, " \t", StringTokenizer::TOK_TRIM);
				std::string mimeType = tokenizer[0];
				for (StringTokenizer::Iterator it = tokenizer.begin()+1; it != tokenizer.end(); ++it)
				{
					std::string ext = Poco::trim(*it);
					if (!ext.empty())
					{
						mimeTypes[ext] = mimeType;
					}
				}
			}
		}
		fclose(f);
	}
}


std::string MiniWebServer::getMimeType(std::string extension)
{
	std::map<std::string, std::string>::iterator mime;
	mime = mimeTypes.find(extension);
	if (mime != mimeTypes.end())
		return mimeTypes[extension];
	else
		return "text/plain";

}

