/*
 * mini web server
 * Copyright 2014, Alex Sualdea.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 *
 *  1. Redistributions of source code must retain the above copyright notice,
 *     this list of conditions and the following disclaimer.
 *  2. Redistributions in binary form must reproduce the above copyright notice,
 *     this list of conditions and the following disclaimer in the documentation
 *     and/or other materials provided with the distribution.
 *  3. The name of the author may not be used to endorse or promote products
 *     derived from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR ``AS IS'' AND ANY EXPRESS OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF
 * MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO
 * EVENT SHALL THE AUTHOR BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS;
 * OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY,
 * WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR
 * OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF
 * ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */
#ifndef _MINI_WEB_SERVER_H
#define _MINI_WEB_SERVER_H

#include "Poco/Net/HTTPServer.h"
#include "Poco/Net/HTTPRequestHandler.h"
#include "Poco/Net/HTTPRequestHandlerFactory.h"
#include "Poco/Net/HTTPServerParams.h"
#include "Poco/Net/HTTPServerRequest.h"
#include "Poco/Net/HTTPServerResponse.h"
#include "Poco/Net/HTTPServerParams.h"
#include "Poco/Net/ServerSocket.h"
#include "Poco/Timestamp.h"
#include "Poco/DateTimeFormatter.h"
#include "Poco/DateTimeFormat.h"
#include "Poco/Exception.h"
#include "Poco/ThreadPool.h"
#include "Poco/Util/ServerApplication.h"
#include "Poco/Util/Option.h"
#include "Poco/Util/OptionSet.h"
#include "Poco/Util/HelpFormatter.h"
#include "Poco/URI.h"
#include "HTTPStreamServer.h"
#include "Settings.h"
#include <memory>


#define MAX_WEB_THREADS 1000
#define SOFTWARE_VERSION "0.5"
#define SERVER_NAME "alx WebServer"
#define mimeTypesDefaultFile "mime.conf"



class MiniWebService
{
public:
	virtual void exec(const Poco::Net::HTTPServerRequest& request,
			  std::map<std::string, std::string> &query, 
			  Poco::Net::HTTPServerResponse& response, 
			  Poco::Net::StreamSocket& sock, 
			  Poco::Net::HTTPServerSession& session) = 0;

	virtual ~MiniWebService() {}
};



//
// execs shell commands
// example http://localhots:8888/sh?ls
// it's insecure and dangerous, but useful for local web servers
//
class WebShellService : public MiniWebService
{

	std::string bash_it(const std::string& cmdline);


public:
	virtual void exec(const Poco::Net::HTTPServerRequest& request,
			  std::map<std::string, std::string> &query,
			  Poco::Net::HTTPServerResponse& response,
			  Poco::Net::StreamSocket& sock,
			  Poco::Net::HTTPServerSession& session)
	{

			Poco::URI u(request.getURI());
			std::ostream& ostr = response.send();
			ostr << bash_it(u.getQuery());
			ostr.flush();
	}

	static std::string name() { return "sh"; }

	virtual ~WebShellService() {}

};


// sample web service
#if 0
class NewService : public MiniWebService
{
public:
	virtual void exec(const Poco::Net::HTTPServerRequest& request,
						std::map<std::string, std::string> &query,
						Poco::Net::HTTPServerResponse& response,
						Poco::Net::StreamSocket& sock,
						Poco::Net::HTTPServerSession& session)
	{
		std::ostream& ostr = response.send();
		ostr << "it works.";
		ostr.flush();
	}

	static std::string name() { return "newservice"; }
};
#endif


class MiniWebRequestHandlerFactory;

// Servidor de paginas HTML
class MiniWebRequestHandler: public HTTPStreamServerRequestHandler
{
	// asks for authentication
	// if return value is false, abort, continue otherwise
	// IMPORTANT: by now it's fake, always return true
	bool getAuthentication(const Poco::Net::HTTPServerRequest& request, Poco::Net::HTTPServerResponse& response);
	// fills the vector 'segments' with the request segments
	void getPathSegmentsFromURI(const std::string& path, std::vector<std::string>& segments);

public:
	MiniWebRequestHandler(MiniWebRequestHandlerFactory* parent_in) : parent(parent_in), docRoot(Settings::get_instance().docroot)
	{

	}
	// switchs between service, php or regular doc
	void handleRequest(Poco::Net::HTTPServerRequest& request, Poco::Net::HTTPServerResponse& response, Poco::Net::StreamSocket& sock, Poco::Net::HTTPServerSession& session);
	// send regular file
	void sendDocument(Poco::Net::HTTPServerResponse& response, const std::vector<std::string>& pathSegments, std::string pathTranslated);
	// executes php and sends php output
	void sendPHP(const Poco::Net::HTTPServerRequest& request, Poco::Net::HTTPServerResponse& response, const std::vector<std::string> pathSegments, const std::string pathTranslated);
	// sends an error page with status
	void sendErrorPage(Poco::Net::HTTPServerResponse& response, Poco::Net::HTTPResponse::HTTPStatus status);
	// executes webservice
	// finds for service name in request and, if found, executes it, otherwise returns 'bad request' or 'not found'
	void webService(Poco::Net::HTTPServerRequest& request, Poco::Net::HTTPServerResponse& response, Poco::Net::StreamSocket& sock, Poco::Net::HTTPServerSession& session);

	std::vector<std::string> pathSegments;
	Poco::URI uri;
	std::string query;
	MiniWebRequestHandlerFactory* parent;
	std::string docRoot;
};

class MiniWebRequestHandlerFactory: public HTTPStreamServerRequestHandlerFactory
{
protected:
	friend class MiniWebRequestHandler;
	class MiniWebServiceList
	{

		std::map<std::string, std::shared_ptr<MiniWebService>> services;

	public:

		void addService(const std::string& svcname, std::shared_ptr<MiniWebService> svc)
		{
			services[svcname] = svc;
		}

		std::shared_ptr<MiniWebService> getService(std::string name)
		{
			if (services.find(name) != services.end())
				return services[name];
			else
				return nullptr;
		}

		bool existsService(std::string name)
		{
			return services.find(name) != services.end();
		}

		void removeAll()
		{
			services.clear();
		}

		void removeService(const std::string& svcname)
		{
			auto it = services.find(svcname);
			if (it != services.end())
				services.erase(it);
		}

	};
	MiniWebServiceList svcs;

public:
	MiniWebRequestHandlerFactory();
	~MiniWebRequestHandlerFactory();
	HTTPStreamServerRequestHandler* createRequestHandler(const Poco::Net::HTTPServerRequest& request);
	void addService(const std::string& svcname, std::shared_ptr<MiniWebService> svc)
	{
		svcs.addService(svcname, svc);
	}
	void removeService(const std::string& svcname)
	{
		svcs.removeService(svcname);
	}
};


#define mimeTypesDefaultFile "mime.conf"


/**
 * Simple web server
 * it serves documents, php and services
 * a service has the form: server:port/service_name?param=value
 */
class MiniWebServer
{
public:
	MiniWebServer();
	virtual ~MiniWebServer();
	bool start(unsigned int port, bool localOnly = true);
	void stop();
	bool started;

	std::shared_ptr<Poco::Net::ServerSocket> getServer() { return svs; }

	void addService(const std::string& svcname, std::shared_ptr<MiniWebService> svc);
	void removeService(const std::string& svcname);

	static std::string getMimeType(std::string extension);
	void loadMimeTypes(std::string mimeTypesFile);

private:
	std::shared_ptr<Poco::Net::ServerSocket> svs;
	HTTPStreamServer *srv;
	bool _localOnly; // if true, listens on 127.0.0.1
	MiniWebRequestHandlerFactory* requestFactory;
	static std::map<std::string, std::string> mimeTypes;
	std::string docRoot;

};






#endif // _MINI_WEB_SERVER_H

